@@ -1339,9 +1339,9 @@ void Scene::removeBuffer()
 
 void Scene::setVertexMax(float x, float y, float z)
 {
-    mVertexMax.x = std::max(x, mVertexMax.x);
-    mVertexMax.y = std::max(y, mVertexMax.y);
-    mVertexMax.z = std::max(z, mVertexMax.z);
+    mVertexMax.x = x;
+    mVertexMax.y = y;
+    mVertexMax.z = z;
 }
 
 glm::vec3& Scene::getVertexMax()
@@ -1351,9 +1351,9 @@ glm::vec3& Scene::getVertexMax()
 
 void Scene::setVertexMin(float x, float y, float z)
 {
-    mVertexMin.x = std::min(x, mVertexMin.x);
-    mVertexMin.y = std::min(y, mVertexMin.y);
-    mVertexMin.z = std::min(z, mVertexMin.z);
+    mVertexMin.x = x;
+    mVertexMin.y = y;
+    mVertexMin.z = z;
 }
 
 glm::vec3& Scene::getVertexMin()
@@ -1384,11 +1384,11 @@ int Scene::loadTexture(const std::string& imagePath, const std::vector<glTFFile>
 {
     if (findTexture(imagePath) == NULL)
     {
-        Texture* oneTexture = new Texture();
-
         const glTFFile* pglTFFile = getGltfFileByFileName(imagePath, inputFiles);
         if( !pglTFFile || pglTFFile->imagewidth == 0 || pglTFFile->imageheight == 0 )
             return LIBGLTF_FILE_NOT_LOAD;
+
+        Texture* oneTexture = new Texture();
         oneTexture->loadTexture2DFromBuffer((unsigned char*)
                                             (pglTFFile->buffer),
                                             pglTFFile->imagewidth,
