@@ -369,7 +369,7 @@ std::ostream &operator<<(std::ostream &o, Token const &tok)
 //! low level internal: main text zone
 struct DSET : public ClarisWksStruct::DSET {
   //! constructor
-  DSET(ClarisWksStruct::DSET const &dset = ClarisWksStruct::DSET()) :
+  explicit DSET(ClarisWksStruct::DSET const &dset = ClarisWksStruct::DSET()) :
     ClarisWksStruct::DSET(dset), m_zones(), m_numChar(0), m_numTextZone(0), m_numParagInfo(0),
     m_numFont(0), m_fatherId(0), m_unknown(0),
     m_subSectionPosList(), m_fontList(), m_paragraphList(), m_tokenList(),
@@ -1270,7 +1270,7 @@ bool ClarisDrawText::sendText(ClarisDrawTextInternal::DSET const &zone, int subZ
             case 2: {
               MWAW_DEBUG_MSG(("ClarisDrawText::sendText: find section token\n"));
               f << "##";
-              listener->insertUnicodeString("1");
+              listener->insertUnicodeString(librevenge::RVNGString("1"));
               break;
             }
             case 3:
