@@ -110,7 +110,7 @@ struct DocZoneStruct {
 //! Internal: the sidebar of a FullWrtParser
 struct SideBar : public FullWrtStruct::ZoneHeader {
   //! constructor
-  SideBar(FullWrtStruct::ZoneHeader &): m_box(), m_page(0)
+  explicit SideBar(FullWrtStruct::ZoneHeader &): m_box(), m_page(0)
   {
   }
   //! the position (in point)
