@@ -64,7 +64,7 @@ namespace MWAWPresentationListenerInternal
 /** the global graphic state of MWAWPresentationListener */
 struct GraphicState {
   //! constructor
-  GraphicState(std::vector<MWAWPageSpan> const &pageList) :
+  explicit GraphicState(std::vector<MWAWPageSpan> const &pageList) :
     m_pageList(pageList), m_metaData(),
     m_isDocumentStarted(false), m_isPageSpanOpened(false), m_isMasterPageSpanOpened(false), m_isAtLeastOnePageOpened(false),
     m_isHeaderFooterStarted(false), m_pageSpan(), m_sentListMarkers(), m_subDocuments()
@@ -366,54 +366,18 @@ void MWAWPresentationListener::insertField(MWAWField const &field)
     MWAW_DEBUG_MSG(("MWAWPresentationListener::setParagraph: called outside a text zone\n"));
     return;
   }
-  switch (field.m_type) {
-  case MWAWField::None:
-    break;
-  case MWAWField::PageCount:
-  case MWAWField::PageNumber:
-  case MWAWField::Title: {
+  librevenge::RVNGPropertyList propList;
+  if (field.addTo(propList)) {
     _flushText();
     _openSpan();
-    librevenge::RVNGPropertyList propList;
-    if (field.m_type==MWAWField::Title)
-      propList.insert("librevenge:field-type", "text:title");
-    else {
-      propList.insert("style:num-format", libmwaw::numberingTypeToString(field.m_numberingType).c_str());
-      if (field.m_type == MWAWField::PageNumber)
-        propList.insert("librevenge:field-type", "text:page-number");
-      else
-        propList.insert("librevenge:field-type", "text:page-count");
-    }
     m_documentInterface->insertField(propList);
-    break;
+    return;
   }
-  case MWAWField::Database:
-    if (field.m_data.length())
-      MWAWPresentationListener::insertUnicodeString(field.m_data.c_str());
-    else
-      MWAWPresentationListener::insertUnicodeString("#DATAFIELD#");
-    break;
-  case MWAWField::Date:
-  case MWAWField::Time: {
-    std::string format(field.m_DTFormat);
-    if (format.length()==0) {
-      if (field.m_type==MWAWField::Date)
-        format="%m/%d/%y";
-      else
-        format="%I:%M:%S %p";
-    }
-    time_t now = time(0L);
-    struct tm timeinfo;
-    if (localtime_r(&now, &timeinfo)) {
-      char buf[256];
-      strftime(buf, 256, format.c_str(), &timeinfo);
-      MWAWPresentationListener::insertUnicodeString(librevenge::RVNGString(buf));
-    }
-    break;
-  }
-  default:
+  librevenge::RVNGString text=field.getString();
+  if (!text.empty())
+    MWAWPresentationListener::insertUnicodeString(text);
+  else {
     MWAW_DEBUG_MSG(("MWAWPresentationListener::insertField: must not be called with type=%d\n", int(field.m_type)));
-    break;
   }
 }
 
@@ -1000,17 +964,17 @@ void MWAWPresentationListener::insertNote(MWAWNote const &, MWAWSubDocumentPtr &
 // picture/textbox
 ///////////////////
 
-void MWAWPresentationListener::insertPicture
+void MWAWPresentationListener::insertShape
 (MWAWPosition const &pos, MWAWGraphicShape const &shape, MWAWGraphicStyle const &style)
 {
   if (!m_ds->m_isDocumentStarted) {
-    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertPicture: the document is not started\n"));
+    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertShape: the document is not started\n"));
     return;
   }
   if (!m_ds->m_isPageSpanOpened)
     _openPageSpan();
   if (m_ps->m_isFrameOpened) {
-    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertPicture: a frame is already open\n"));
+    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertShape: a frame is already open\n"));
     return;
   }
 
@@ -1036,13 +1000,12 @@ void MWAWPresentationListener::insertPicture
   case MWAWGraphicShape::C_Bad:
     break;
   default:
-    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertPicture: unexpected shape\n"));
+    MWAW_DEBUG_MSG(("MWAWPresentationListener::insertShape: unexpected shape\n"));
     break;
   }
 }
 
-void MWAWPresentationListener::insertPicture
-(MWAWPosition const &pos, const librevenge::RVNGBinaryData &binaryData, std::string type, MWAWGraphicStyle const &style)
+void MWAWPresentationListener::insertPicture(MWAWPosition const &pos, MWAWEmbeddedObject const &picture, MWAWGraphicStyle const &style)
 {
   if (!m_ds->m_isDocumentStarted) {
     MWAW_DEBUG_MSG(("MWAWPresentationListener::insertPicture: the document is not started\n"));
@@ -1072,9 +1035,8 @@ void MWAWPresentationListener::insertPicture
     list.insert("librevenge:rotate-cx",center[0], librevenge::RVNG_POINT);
     list.insert("librevenge:rotate-cy",center[1], librevenge::RVNG_POINT);
   }
-  list.insert("librevenge:mime-type", type.c_str());
-  list.insert("office:binary-data", binaryData);
-  m_documentInterface->drawGraphicObject(list);
+  if (picture.addTo(list))
+    m_documentInterface->drawGraphicObject(list);
 }
 
 void MWAWPresentationListener::insertTextBox
@@ -1379,9 +1341,9 @@ void  MWAWPresentationListener::closeGroup()
     MWAW_DEBUG_MSG(("MWAWPresentationListener::closeGroup: called but no group is already opened\n"));
     return;
   }
-  m_documentInterface->closeGroup();
   _endSubDocument();
   _popParsingState();
+  m_documentInterface->closeGroup();
 }
 
 bool MWAWPresentationListener::openLayer(librevenge::RVNGString const &layerName)
@@ -1491,6 +1453,10 @@ void MWAWPresentationListener::_handleFrameParameters(librevenge::RVNGPropertyLi
     list.insert("style:wrap", "run-through");
     list.insert("style:run-through", "foreground");
   }
+  else if (pos.m_wrapping ==  MWAWPosition::WParallel) {
+    list.insert("style:wrap", "parallel");
+    list.insert("style:run-through", "foreground");
+  }
   else if (pos.m_wrapping ==  MWAWPosition::WRunThrough)
     list.insert("style:wrap", "run-through");
   else
