@@ -107,11 +107,9 @@ struct Zone {
   }
 
   //! return a binary data (if known)
-  virtual bool getBinaryData(MWAWInputStreamPtr,
-                             librevenge::RVNGBinaryData &res, std::string &pictType) const
+  virtual bool getBinaryData(MWAWInputStreamPtr, MWAWEmbeddedObject &picture) const
   {
-    res.clear();
-    pictType="";
+    picture=MWAWEmbeddedObject();
     return false;
   }
 
@@ -293,7 +291,7 @@ void Zone::print(std::ostream &o) const
 //! Internal: the group of a MsWksGraph
 struct GroupZone : public Zone {
   // constructor
-  GroupZone(Zone const &z) : Zone(z), m_childs() { }
+  explicit GroupZone(Zone const &z) : Zone(z), m_childs() { }
 
   //! return the type
   virtual Type type() const
@@ -317,7 +315,7 @@ struct GroupZone : public Zone {
 //! Internal: the simple form of a MsWksGraph ( line, rect, ...)
 struct BasicShape : public Zone {
   //! constructor
-  BasicShape(Zone const &z) : Zone(z), m_shape()
+  explicit BasicShape(Zone const &z) : Zone(z), m_shape()
   {
   }
   //! return the type
@@ -337,7 +335,7 @@ struct BasicShape : public Zone {
     float res=m_style.m_lineWidth;
     if (m_shape.m_type==MWAWGraphicShape::Line) {
       for (int i=0; i<2; ++i) {
-        if (m_style.m_arrows[i]) res+=4;
+        if (!m_style.m_arrows[i].isEmpty()) res+=4;
       }
     }
     return 0.5f*res;
@@ -347,7 +345,7 @@ struct BasicShape : public Zone {
   {
     MWAWGraphicStyle style(m_style);
     if (m_subType!=0)
-      style.m_arrows[0] = style.m_arrows[1]=false;
+      style.m_arrows[0] = style.m_arrows[1]=MWAWGraphicStyle::Arrow();
     return style;
   }
 
@@ -362,7 +360,7 @@ private:
 //! Internal: the table of a MsWksGraph
 struct Chart : public Zone {
   //! constructor
-  Chart(Zone const &z) : Zone(z), m_chartId(0) { }
+  explicit Chart(Zone const &z) : Zone(z), m_chartId(0) { }
   //! empty constructor
   Chart() : Zone(), m_chartId(0) { }
 
@@ -379,7 +377,7 @@ struct Chart : public Zone {
 //! Internal: the picture of a MsWksGraph
 struct DataPict : public Zone {
   //! constructor
-  DataPict(Zone const &z) : Zone(z), m_dataEndPos(-1), m_naturalBox() { }
+  explicit DataPict(Zone const &z) : Zone(z), m_dataEndPos(-1), m_naturalBox() { }
   //! empty constructor
   DataPict() : Zone(), m_dataEndPos(-1), m_naturalBox() { }
 
@@ -389,8 +387,7 @@ struct DataPict : public Zone {
     return Pict;
   }
   //! return a binary data (if known)
-  virtual bool getBinaryData(MWAWInputStreamPtr ip,
-                             librevenge::RVNGBinaryData &res, std::string &type) const;
+  virtual bool getBinaryData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture) const;
 
   //! operator<<
   virtual void print(std::ostream &o) const
@@ -403,11 +400,9 @@ struct DataPict : public Zone {
   mutable MWAWBox2f m_naturalBox;
 };
 
-bool DataPict::getBinaryData(MWAWInputStreamPtr ip,
-                             librevenge::RVNGBinaryData &data, std::string &pictType) const
+bool DataPict::getBinaryData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture) const
 {
-  data.clear();
-  pictType="";
+  picture=MWAWEmbeddedObject();
   long endPos = m_dataEndPos<=0 ? m_pos.end() : m_dataEndPos;
   long pictSize = endPos-m_dataPos;
   if (pictSize < 0) {
@@ -437,14 +432,14 @@ bool DataPict::getBinaryData(MWAWInputStreamPtr ip,
   ip->seek(m_dataPos, librevenge::RVNG_SEEK_SET);
   shared_ptr<MWAWPict> pict(MWAWPictData::get(ip, (int)pictSize));
 
-  return pict && pict->getBinary(data,pictType);
+  return pict && pict->getBinary(picture);
 }
 
 ////////////////////////////////////////
 //! Internal: the bitmap of a MsWksGraph
 struct DataBitmap : public Zone {
   //! constructor
-  DataBitmap(Zone const &z) : Zone(z), m_numRows(0), m_numCols(0), m_dataSize(0),
+  explicit DataBitmap(Zone const &z) : Zone(z), m_numRows(0), m_numCols(0), m_dataSize(0),
     m_naturalBox() { }
   //! empty constructor
   DataBitmap() : Zone(), m_numRows(0), m_numCols(0), m_dataSize(0), m_naturalBox() { }
@@ -455,8 +450,7 @@ struct DataBitmap : public Zone {
     return Bitmap;
   }
   //! return a binary data (if known)
-  bool getPictureData(MWAWInputStreamPtr ip, librevenge::RVNGBinaryData &res,
-                      std::string &type, std::vector<MWAWColor> const &palette) const;
+  bool getPictureData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture, std::vector<MWAWColor> const &palette) const;
 
   //! operator<<
   virtual void print(std::ostream &o) const
@@ -474,12 +468,9 @@ struct DataBitmap : public Zone {
   mutable MWAWBox2f m_naturalBox;
 };
 
-bool DataBitmap::getPictureData
-(MWAWInputStreamPtr ip, librevenge::RVNGBinaryData &data, std::string &pictType,
- std::vector<MWAWColor> const &palette) const
+bool DataBitmap::getPictureData(MWAWInputStreamPtr ip, MWAWEmbeddedObject &picture, std::vector<MWAWColor> const &palette) const
 {
-  data.clear();
-  pictType="";
+  picture=MWAWEmbeddedObject();
   if (m_dataSize <= 0 || m_dataSize < m_numRows*m_numCols) {
     MWAW_DEBUG_MSG(("MsWksGraphInternal::DataBitmap::getPictureData: dataSize size is bad\n"));
     return false;
@@ -502,14 +493,14 @@ bool DataBitmap::getPictureData
     pos += szCol;
   }
 
-  return pict->getBinary(data,pictType);
+  return pict->getBinary(picture);
 }
 
 ////////////////////////////////////////
 //! Internal: the table of a MsWksGraph
 struct Table : public Zone {
   //! constructor
-  Table(Zone const &z) : Zone(z), m_tableId(0) { }
+  explicit Table(Zone const &z) : Zone(z), m_tableId(0) { }
   //! empty constructor
   Table() : Zone(), m_tableId(0) { }
 
@@ -526,7 +517,7 @@ struct Table : public Zone {
 //! Internal: the textbox of a MsWksGraph ( v2-v3)
 struct TextBox : public Zone {
   //! constructor
-  TextBox(Zone const &z) :
+  explicit TextBox(Zone const &z) :
     Zone(z), m_numPositions(-1), m_fontsList(), m_positions(), m_formats(), m_text(""), m_justify(MWAWParagraph::JustificationLeft)
   { }
 
@@ -588,7 +579,7 @@ private:
 //! Internal: the ole zone of a MsWksGraph ( v4)
 struct OLEZone : public Zone {
   //! constructor
-  OLEZone(Zone const &z) : Zone(z), m_oleId(-1), m_dim()
+  explicit OLEZone(Zone const &z) : Zone(z), m_oleId(-1), m_dim()
   { }
 
   //! return the type
@@ -614,7 +605,7 @@ struct OLEZone : public Zone {
 //! Internal: the textbox of a MsWksGraph ( v4)
 struct TextBoxv4 : public Zone {
   //! constructor
-  TextBoxv4(Zone const &z) : Zone(z), m_text(), m_frame("")
+  explicit TextBoxv4(Zone const &z) : Zone(z), m_text(), m_frame("")
   { }
 
   //! return the type
@@ -670,7 +661,7 @@ struct Patterns {
   //! return the pattern corresponding to an id
   bool get(int id, MWAWGraphicStyle::Pattern &pat) const
   {
-    if (id < 0 || id > m_num) {
+    if (id < 0 || id >= m_num) {
       MWAW_DEBUG_MSG(("MsWksGraphInternal::Patterns::get: can not find pattern %d\n", id));
       return false;
     }
@@ -684,7 +675,7 @@ struct Patterns {
   //! return the percentage corresponding to a pattern
   float getPercent(int id) const
   {
-    if (id < 0 || id > m_num) {
+    if (id < 0 || id >= m_num) {
       MWAW_DEBUG_MSG(("MsWksGraphInternal::Patterns::getPatternPercent: can not find pattern %d\n", id));
       return 1.0;
     }
@@ -1056,9 +1047,9 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
       if (i==0)
         style.m_lineColor=MWAWColor::barycenter(percent, style.m_baseLineColor, 1.f-percent, style.m_baseSurfaceColor);
       else if (m_state->getPattern(pattern, pId)) {
-        style.m_pattern=pattern;
-        style.m_pattern.m_colors[0] = style.m_baseSurfaceColor;
-        style.m_pattern.m_colors[1] = style.m_baseLineColor;
+        pattern.m_colors[0] = style.m_baseSurfaceColor;
+        pattern.m_colors[1] = style.m_baseLineColor;
+        style.setPattern(pattern);
       }
       else
         style.setSurfaceColor(MWAWColor::barycenter(percent, style.m_baseLineColor, 1.f-percent, style.m_baseSurfaceColor));
@@ -1119,9 +1110,9 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
         else if (m_state->getPattern(pattern, patId, rsid)) {
           percent = m_state->getPatternPercent(patId, rsid);
           if (i) {
-            style.m_pattern=pattern;
-            style.m_pattern.m_colors[0] = style.m_baseSurfaceColor;
-            style.m_pattern.m_colors[1] = style.m_baseLineColor;
+            pattern.m_colors[0] = style.m_baseSurfaceColor;
+            pattern.m_colors[1] = style.m_baseLineColor;
+            style.setPattern(pattern);
             done = true;
           }
         }
@@ -1187,9 +1178,9 @@ bool MsWksGraph::readPictHeader(MsWksGraphInternal::Zone &pict)
   int lineFlags = (int) input->readULong(1);
   switch (lineFlags&3) {
   case 2:
-    style.m_arrows[0]=true;
+    style.m_arrows[0]=MWAWGraphicStyle::Arrow::plain();
   case 1:
-    style.m_arrows[1]=true;
+    style.m_arrows[1]=MWAWGraphicStyle::Arrow::plain();
     break;
   default:
     f << "#arrow=3,";
@@ -2213,10 +2204,9 @@ void MsWksGraph::sendGroup(int id, MWAWPosition const &pos)
   graphicListener->startDocument();
   sendGroup(group, graphicListener);
   graphicListener->endDocument();
-  librevenge::RVNGBinaryData data;
-  std::string type;
-  if (graphicEncoder.getBinaryResult(data,type))
-    listener->insertPicture(pos, data, type);
+  MWAWEmbeddedObject picture;
+  if (graphicEncoder.getBinaryResult(picture))
+    listener->insertPicture(pos, picture);
 }
 
 void MsWksGraph::sendGroupChild(int id, MWAWPosition const &pos)
@@ -2292,7 +2282,7 @@ void MsWksGraph::sendGroupChild(int id, MWAWPosition const &pos)
           sendGroup(static_cast<MsWksGraphInternal::GroupZone const &>(localChild), graphicListener);
         else if (localChild.type()==MsWksGraphInternal::Zone::Shape) {
           MsWksGraphInternal::BasicShape const &shape=static_cast<MsWksGraphInternal::BasicShape const &>(localChild);
-          graphicListener->insertPicture(pictPos, shape.m_shape, shape.getStyle());
+          graphicListener->insertShape(pictPos, shape.m_shape, shape.getStyle());
         }
         else if (localChild.type()==MsWksGraphInternal::Zone::Text) {
           shared_ptr<MsWksGraphInternal::SubDocument> subdoc
@@ -2304,12 +2294,11 @@ void MsWksGraph::sendGroupChild(int id, MWAWPosition const &pos)
         }
       }
       graphicListener->endDocument();
-      librevenge::RVNGBinaryData data;
-      std::string type;
-      if (graphicEncoder.getBinaryResult(data,type)) {
+      MWAWEmbeddedObject picture;
+      if (graphicEncoder.getBinaryResult(picture)) {
         partialPos.setOrigin(pos.origin()+partialBdBox[0]-group.m_box[0]);
         partialPos.setSize(partialBdBox.size());
-        listener->insertPicture(partialPos, data, type);
+        listener->insertPicture(partialPos, picture);
         if (isLast)
           break;
         childNotSent=c;
@@ -2377,7 +2366,7 @@ void MsWksGraph::sendGroup(MsWksGraphInternal::GroupZone const &group, MWAWGraph
       sendGroup(static_cast<MsWksGraphInternal::GroupZone const &>(child), listener);
     else if (child.type()==MsWksGraphInternal::Zone::Shape) {
       MsWksGraphInternal::BasicShape const &shape=static_cast<MsWksGraphInternal::BasicShape const &>(child);
-      listener->insertPicture(pictPos, shape.m_shape, shape.getStyle());
+      listener->insertShape(pictPos, shape.m_shape, shape.getStyle());
     }
     else if (child.type()==MsWksGraphInternal::Zone::Text) {
       shared_ptr<MsWksGraphInternal::SubDocument> subdoc
@@ -2455,7 +2444,7 @@ bool MsWksGraph::readText(MsWksGraphInternal::TextBox &textBox)
 
     if (fPos < 0 || form < -1) return false;
     if ((form == -1 && i != textBox.m_numPositions) ||
-        (i && fPos < textBox.m_positions[(size_t) i-1])) {
+        (!textBox.m_positions.empty() && fPos < textBox.m_positions.back())) {
       MWAW_DEBUG_MSG(("MsWksGraph::readText: find odd positions\n"));
       f << "#";
       continue;
@@ -2737,10 +2726,9 @@ void MsWksGraph::send(int id, MWAWPosition const &pos)
     textPos.m_wrapping=pos.m_wrapping;
     graphicListener.insertTextBox(textPos, subdoc, style);
     graphicListener.endDocument();
-    librevenge::RVNGBinaryData data;
-    std::string type;
-    if (graphicEncoder.getBinaryResult(data, type))
-      listener->insertPicture(pictPos, data, type);
+    MWAWEmbeddedObject picture;
+    if (graphicEncoder.getBinaryResult(picture))
+      listener->insertPicture(pictPos, picture);
     return;
   }
   case MsWksGraphInternal::Zone::TableZone: {
@@ -2768,25 +2756,23 @@ void MsWksGraph::send(int id, MWAWPosition const &pos)
     return;
   case MsWksGraphInternal::Zone::Bitmap: {
     MsWksGraphInternal::DataBitmap &bmap = static_cast<MsWksGraphInternal::DataBitmap &>(*zone);
-    librevenge::RVNGBinaryData data;
-    std::string type;
-    if (!bmap.getPictureData(input, data,type,m_document.getPalette(4)))
+    MWAWEmbeddedObject picture;
+    if (!bmap.getPictureData(input, picture,m_document.getPalette(4)))
       break;
     m_document.ascii().skipZone(bmap.m_dataPos, bmap.m_pos.end()-1);
-    listener->insertPicture(pictPos, data, type, zone->m_style);
+    listener->insertPicture(pictPos, picture, zone->m_style);
     return;
   }
   case MsWksGraphInternal::Zone::Shape: {
     MsWksGraphInternal::BasicShape &shape = static_cast<MsWksGraphInternal::BasicShape &>(*zone);
-    listener->insertPicture(pictPos, shape.m_shape, shape.getStyle());
+    listener->insertShape(pictPos, shape.m_shape, shape.getStyle());
     return;
   }
   case MsWksGraphInternal::Zone::Pict: {
-    librevenge::RVNGBinaryData data;
-    std::string type;
-    if (!zone->getBinaryData(input, data,type))
+    MWAWEmbeddedObject picture;
+    if (!zone->getBinaryData(input, picture))
       break;
-    listener->insertPicture(pictPos, data, type, zone->m_style);
+    listener->insertPicture(pictPos, picture, zone->m_style);
     return;
   }
   case MsWksGraphInternal::Zone::Textv4: {
