@@ -49,7 +49,7 @@ public:
   void collectEllipse(unsigned id, unsigned level, double cx, double cy, double xleft, double yleft, double xtop, double ytop);
   void collectLine(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
                    const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                   const boost::optional<unsigned char> &lineCap);
+                   const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding);
   void collectFillAndShadow(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
                             const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
                             const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
@@ -109,10 +109,15 @@ public:
   void collectParaIX(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
                      const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
                      const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
+                     const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr, const boost::optional<VSDName> &bulletFont,
+                     const boost::optional<double> &bulletFontSize, const boost::optional<double> &textPosAfterBullet,
                      const boost::optional<unsigned> &flags);
   void collectDefaultParaStyle(unsigned charCount, const boost::optional<double> &indFirst, const boost::optional<double> &indLeft,
                                const boost::optional<double> &indRight, const boost::optional<double> &spLine, const boost::optional<double> &spBefore,
-                               const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align, const boost::optional<unsigned> &flags);
+                               const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
+                               const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
+                               const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
+                               const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags);
   void collectTextBlock(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
                         const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
                         const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
@@ -122,13 +127,15 @@ public:
   void collectName(unsigned id, unsigned level,  const librevenge::RVNGBinaryData &name, TextFormat format);
   void collectPageSheet(unsigned id, unsigned level);
   void collectMisc(unsigned level, const VSDMisc &misc);
-
+  void collectLayer(unsigned id, unsigned level, const VSDLayer &layer);
+  void collectLayerMem(unsigned level, const VSDName &layerMem);
+  void collectTabsDataList(unsigned level, const std::map<unsigned, VSDTabSet> &tabSets);
 
   // Style collectors
   void collectStyleSheet(unsigned id, unsigned level, unsigned parentLineStyle, unsigned parentFillStyle, unsigned parentTextStyle);
   void collectLineStyle(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
                         const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                        const boost::optional<unsigned char> &lineCap);
+                        const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding);
   void collectFillStyle(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
                         const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
                         const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
@@ -146,7 +153,9 @@ public:
   void collectParaIXStyle(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
                           const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
                           const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
-                          const boost::optional<unsigned> &flags);
+                          const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
+                          const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
+                          const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags);
   void collectTextBlockStyle(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
                              const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
                              const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
@@ -155,6 +164,8 @@ public:
   void collectStyleThemeReference(unsigned level, const boost::optional<long> &lineColour, const boost::optional<long> &fillColour,
                                   const boost::optional<long> &shadowColour, const boost::optional<long> &fontColour);
 
+  virtual void collectMetaData(const librevenge::RVNGPropertyList &metaData);
+
 
   // Field list
   void collectFieldList(unsigned id, unsigned level);
@@ -212,6 +223,14 @@ private:
   void _outputCubicBezierSegment(const std::vector<std::pair<double, double> > &points);
   void _outputQuadraticBezierSegment(const std::vector<std::pair<double, double> > &points);
   void _outputLinearBezierSegment(const std::vector<std::pair<double, double> > &points);
+  void _appendVisibleAndPrintable(librevenge::RVNGPropertyList &propList);
+  void _bulletFromParaFormat(VSDBullet &bullet, const VSDParaStyle &paraStyle);
+  void _listLevelFromBullet(librevenge::RVNGPropertyList &propList, const VSDBullet &bullet);
+  void _fillParagraphProperties(librevenge::RVNGPropertyList &propList, const VSDParaStyle &style);
+  void _fillTabSet(librevenge::RVNGPropertyList &propList, const VSDTabSet &tabSet);
+  void _fillCharProperties(librevenge::RVNGPropertyList &propList, const VSDCharStyle &style);
+  void _convertToPath(const std::vector<librevenge::RVNGPropertyList> &segmentVector,
+                      librevenge::RVNGPropertyListVector &path, double rounding);
 
   bool m_isPageStarted;
   double m_pageWidth;
@@ -258,12 +277,11 @@ private:
 
   std::map<unsigned, NURBSData> m_NURBSData;
   std::map<unsigned, PolylineData> m_polylineData;
-  librevenge::RVNGBinaryData m_textStream;
+  librevenge::RVNGString m_currentText;
   std::map<unsigned, librevenge::RVNGString> m_names, m_stencilNames;
   std::vector<librevenge::RVNGString> m_fields;
   VSDFieldList m_stencilFields;
   unsigned m_fieldIndex;
-  TextFormat m_textFormat;
   std::vector<VSDCharStyle> m_charFormats;
   std::vector<VSDParaStyle> m_paraFormats;
 
@@ -290,6 +308,8 @@ private:
   VSDPage m_currentPage;
   VSDPages m_pages;
 
+  VSDLayerList m_layerList;
+
   std::vector<std::pair<double, double> > m_splineControlPoints;
   std::vector<double> m_splineKnotVector;
   double m_splineX, m_splineY;
@@ -298,6 +318,10 @@ private:
   unsigned m_splineLevel;
   unsigned m_currentShapeLevel;
   bool m_isBackgroundPage;
+
+  VSDLayerList m_currentLayerList;
+  std::vector<unsigned> m_currentLayerMem;
+  std::vector<VSDTabSet> m_tabSets;
 };
 
 } // namespace libvisio
