@@ -1,29 +1,9 @@
-/*************************************************************************
- *
- * Copyright (c) 2012 Markus Mohrhard
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
- *
- ************************************************************************/
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/*
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ */
 
 #include "gnumeric_helper.hpp"
 
@@ -48,16 +28,17 @@ std::vector<pstring> split_string(pstring str, char sep)
         if (str[i] == sep)
         {
             ret.push_back( pstring( start, len ) );
-            len = 0;
 
             // if not at the end move the start string
             if (i < n-1)
                 start = start + len + 1;
 
+            len = 0;
         }
         else
             ++len;
     }
+    ret.push_back( pstring( start, len ) );
 
     return ret;
 }
@@ -89,3 +70,4 @@ bool gnumeric_helper::parse_RGB_color_attribute(color_elem_t& red, color_elem_t&
 }
 
 }
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
